#![no_main]
#![no_std]

use algorithms::crc::{Crc, Crc32 as Crc32Rust, ffi::Crc32 as Crc32C};

use defmt::{debug, error, info};
use panic_probe as _;

use defmt_rtt as _; // global logger

use cortex_m_rt::{ExceptionFrame, entry};
use hal::{pac, prelude::*, rcc::Config};

mod hardfaults; // defines a handler for evil hardfaults

// same panicking *behavior* as `panic-probe` but doesn't print a panic message
// this prevents the panic message being printed *twice* when `defmt::panic` is invoked
#[defmt::panic_handler]
fn panic() -> ! {
    cortex_m::asm::udf()
}

const DATA: &[u8] = b"hello world AAAAAAAAAAAAAAAAAAAAAAA";

#[entry]
fn main() -> ! {
    let dp = pac::Peripherals::take().unwrap();
    let cp = cortex_m::Peripherals::take().unwrap();

    // Configure the clock.
    let mut rcc = dp.RCC.freeze(Config::hsi16());

    // Acquire the GPIOA peripheral. This also enables the clock for GPIOA in
    // the RCC register.
    let gpioa = dp.GPIOA.split(&mut rcc);

    // Configure PA5 as output.
    let mut led = gpioa.pa5.into_push_pull_output();

    // Get the delay provider.
    let mut delay = cp.SYST.delay(rcc.clocks);

    debug!("Asserting that libalgorithms is correctly loaded");
    assert!(algorithms::ffi_is_loaded(), "ffi is not loaded?");

    let mut crc;

    #[allow(clippy::never_loop)]
    loop {
        led.set_high().unwrap();
        delay.delay_ms(500_u16);

        info!("Calculating CRCs for: {:?}", DATA);

        debug!("Now calculating Crc32Rust...");
        crc = Crc32Rust::checksum(DATA);
        info!("CRC (Rust): {:08x}", crc);
        debug!("Now calculating Crc32C...");
        crc = Crc32C::checksum(DATA);
        info!("CRC (C)   : {:08x}", crc);

        led.set_low().unwrap();
        delay.delay_ms(500_u16);
    }
}
